@@ -41,7 +41,6 @@ import org.apache.http.Header;
 import org.apache.http.HttpHost;
 import org.apache.http.HttpRequestInterceptor;
 import org.apache.http.HttpResponseInterceptor;
-import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.auth.AuthSchemeProvider;
 import org.apache.http.client.AuthenticationStrategy;
 import org.apache.http.client.CookieStore;
@@ -81,6 +80,7 @@ import org.apache.http.impl.client.DefaultConnectionKeepAliveStrategy;
 import org.apache.http.impl.client.DefaultRedirectStrategy;
 import org.apache.http.impl.client.NoopUserTokenHandler;
 import org.apache.http.impl.client.ProxyAuthenticationStrategy;
+import org.apache.http.impl.client.SystemDefaultCredentialsProvider;
 import org.apache.http.impl.client.TargetAuthenticationStrategy;
 import org.apache.http.impl.conn.DefaultProxyRoutePlanner;
 import org.apache.http.impl.conn.DefaultRoutePlanner;
@@ -142,7 +142,6 @@ import org.apache.http.util.VersionInfo;
  *
  * @since 4.0
  */
-@NotThreadSafe
 public class HttpAsyncClientBuilder {
 
     private NHttpClientConnectionManager connManager;
@@ -649,9 +648,9 @@ public class HttpAsyncClientBuilder {
                 SSLContext sslcontext = this.sslcontext;
                 if (sslcontext == null) {
                     if (systemProperties) {
-                        sslcontext = SSLContexts.createDefault();
-                    } else {
                         sslcontext = SSLContexts.createSystemDefault();
+                    } else {
+                        sslcontext = SSLContexts.createDefault();
                     }
                 }
                 final String[] supportedProtocols = systemProperties ? split(
@@ -666,7 +665,7 @@ public class HttpAsyncClientBuilder {
                         sslcontext, supportedProtocols, supportedCipherSuites, hostnameVerifier);
             }
             final ConnectingIOReactor ioreactor = IOReactorUtils.create(
-                defaultIOReactorConfig != null ? defaultIOReactorConfig : IOReactorConfig.DEFAULT);
+                defaultIOReactorConfig != null ? defaultIOReactorConfig : IOReactorConfig.DEFAULT, threadFactory);
             final PoolingNHttpClientConnectionManager poolingmgr = new PoolingNHttpClientConnectionManager(
                     ioreactor,
                     RegistryBuilder.<SchemeIOSessionStrategy>create()
@@ -833,9 +832,12 @@ public class HttpAsyncClientBuilder {
 
         CredentialsProvider defaultCredentialsProvider = this.credentialsProvider;
         if (defaultCredentialsProvider == null) {
-            defaultCredentialsProvider = new BasicCredentialsProvider();
+            if (systemProperties) {
+                defaultCredentialsProvider = new SystemDefaultCredentialsProvider();
+            } else {
+                defaultCredentialsProvider = new BasicCredentialsProvider();
+            }
         }
-
         RedirectStrategy redirectStrategy = this.redirectStrategy;
         if (redirectStrategy == null) {
             redirectStrategy = DefaultRedirectStrategy.INSTANCE;
