@@ -33,8 +33,10 @@ import java.net.SocketAddress;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -116,7 +118,7 @@ public class PoolingNHttpClientConnectionManager
     public PoolingNHttpClientConnectionManager(
             final ConnectingIOReactor ioreactor,
             final Registry<SchemeIOSessionStrategy> iosessionFactoryRegistry) {
-        this(ioreactor, null, iosessionFactoryRegistry, null);
+        this(ioreactor, null, iosessionFactoryRegistry, (DnsResolver) null);
     }
 
     public PoolingNHttpClientConnectionManager(
@@ -126,17 +128,24 @@ public class PoolingNHttpClientConnectionManager
         this(ioreactor, connFactory, getDefaultRegistry(), dnsResolver);
     }
 
+    public PoolingNHttpClientConnectionManager(
+            final ConnectingIOReactor ioreactor,
+            final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory,
+            final SocketAddressResolver<HttpRoute> socketAddressResolver) {
+        this(ioreactor, connFactory, getDefaultRegistry(), socketAddressResolver);
+    }
+
     public PoolingNHttpClientConnectionManager(
             final ConnectingIOReactor ioreactor,
             final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory) {
-        this(ioreactor, connFactory, getDefaultRegistry(), null);
+        this(ioreactor, connFactory, getDefaultRegistry(), (DnsResolver) null);
     }
 
     public PoolingNHttpClientConnectionManager(
             final ConnectingIOReactor ioreactor,
             final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory,
             final Registry<SchemeIOSessionStrategy> iosessionFactoryRegistry) {
-        this(ioreactor, connFactory, iosessionFactoryRegistry, null);
+        this(ioreactor, connFactory, iosessionFactoryRegistry, (DnsResolver) null);
     }
 
     public PoolingNHttpClientConnectionManager(
@@ -148,6 +157,15 @@ public class PoolingNHttpClientConnectionManager
             -1, TimeUnit.MILLISECONDS);
     }
 
+    public PoolingNHttpClientConnectionManager(
+            final ConnectingIOReactor ioreactor,
+            final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory,
+            final Registry<SchemeIOSessionStrategy> iosessionFactoryRegistry,
+            final SocketAddressResolver<HttpRoute> socketAddressResolver) {
+        this(ioreactor, connFactory, iosessionFactoryRegistry, socketAddressResolver,
+                -1, TimeUnit.MILLISECONDS);
+    }
+
     public PoolingNHttpClientConnectionManager(
             final ConnectingIOReactor ioreactor,
             final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory,
@@ -155,15 +173,26 @@ public class PoolingNHttpClientConnectionManager
             final SchemePortResolver schemePortResolver,
             final DnsResolver dnsResolver,
             final long timeToLive, final TimeUnit tunit) {
+        this(ioreactor, connFactory, iosessionFactoryRegistry,
+                new InternalAddressResolver(schemePortResolver, dnsResolver), timeToLive, tunit);
+    }
+
+    public PoolingNHttpClientConnectionManager(
+            final ConnectingIOReactor ioreactor,
+            final NHttpConnectionFactory<ManagedNHttpClientConnection> connFactory,
+            final Registry<SchemeIOSessionStrategy> iosessionFactoryRegistry,
+            final SocketAddressResolver<HttpRoute> socketAddressResolver,
+            final long timeToLive, final TimeUnit tunit) {
         super();
         Args.notNull(ioreactor, "I/O reactor");
         Args.notNull(iosessionFactoryRegistry, "I/O session factory registry");
+        Args.notNull(socketAddressResolver, "Socket address resolver");
         this.ioreactor = ioreactor;
         this.configData = new ConfigData();
         this.pool = new CPool(ioreactor,
-            new InternalConnectionFactory(this.configData, connFactory),
-            new InternalAddressResolver(schemePortResolver, dnsResolver),
-            2, 20, timeToLive, tunit != null ? tunit : TimeUnit.MILLISECONDS);
+                new InternalConnectionFactory(this.configData, connFactory),
+                socketAddressResolver,
+                2, 20, timeToLive, tunit != null ? tunit : TimeUnit.MILLISECONDS);
         this.iosessionFactoryRegistry = iosessionFactoryRegistry;
     }
 
@@ -249,7 +278,7 @@ public class PoolingNHttpClientConnectionManager
         if (this.log.isDebugEnabled()) {
             this.log.debug("Connection request: " + format(route, state) + formatStats(route));
         }
-        final BasicFuture<NHttpClientConnection> future = new BasicFuture<NHttpClientConnection>(callback);
+        final BasicFuture<NHttpClientConnection> resultFuture = new BasicFuture<NHttpClientConnection>(callback);
         final HttpHost host;
         if (route.getProxyHost() != null) {
             host = route.getProxyHost();
@@ -259,14 +288,73 @@ public class PoolingNHttpClientConnectionManager
         final SchemeIOSessionStrategy sf = this.iosessionFactoryRegistry.lookup(
                 host.getSchemeName());
         if (sf == null) {
-            future.failed(new UnsupportedSchemeException(host.getSchemeName() +
+            resultFuture.failed(new UnsupportedSchemeException(host.getSchemeName() +
                     " protocol is not supported"));
-            return future;
+            return resultFuture;
         }
-        this.pool.lease(route, state,
+        final Future<CPoolEntry> leaseFuture = this.pool.lease(route, state,
                 connectTimeout, leaseTimeout, tunit != null ? tunit : TimeUnit.MILLISECONDS,
-                new InternalPoolEntryCallback(future));
-        return future;
+                new FutureCallback<CPoolEntry>() {
+
+                    @Override
+                    public void completed(final CPoolEntry entry) {
+                        Asserts.check(entry.getConnection() != null, "Pool entry with no connection");
+                        if (log.isDebugEnabled()) {
+                            log.debug("Connection leased: " + format(entry) + formatStats(entry.getRoute()));
+                        }
+                        final NHttpClientConnection managedConn = CPoolProxy.newProxy(entry);
+                        if (!resultFuture.completed(managedConn)) {
+                            pool.release(entry, true);
+                        }
+                    }
+
+                    @Override
+                    public void failed(final Exception ex) {
+                        if (log.isDebugEnabled()) {
+                            log.debug("Connection request failed", ex);
+                        }
+                        resultFuture.failed(ex);
+                    }
+
+                    @Override
+                    public void cancelled() {
+                        log.debug("Connection request cancelled");
+                        resultFuture.cancel(true);
+                    }
+
+                });
+        return new Future<NHttpClientConnection>() {
+
+            @Override
+            public boolean cancel(final boolean mayInterruptIfRunning) {
+                try {
+                    leaseFuture.cancel(mayInterruptIfRunning);
+                } finally {
+                    return resultFuture.cancel(mayInterruptIfRunning);
+                }
+            }
+
+            @Override
+            public boolean isCancelled() {
+                return resultFuture.isCancelled();
+            }
+
+            @Override
+            public boolean isDone() {
+                return resultFuture.isDone();
+            }
+
+            @Override
+            public NHttpClientConnection get() throws InterruptedException, ExecutionException {
+                return resultFuture.get();
+            }
+
+            @Override
+            public NHttpClientConnection get(final long timeout, final TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
+                return resultFuture.get(timeout, unit);
+            }
+
+        };
     }
 
     @Override
@@ -411,6 +499,11 @@ public class PoolingNHttpClientConnectionManager
         this.pool.closeExpired();
     }
 
+    public void validatePendingRequests() {
+        log.debug("Validating pending requests");
+        this.pool.validatePendingRequests();
+    }
+
     @Override
     public int getMaxTotal() {
         return this.pool.getMaxTotal();
@@ -474,44 +567,6 @@ public class PoolingNHttpClientConnectionManager
         this.configData.setConnectionConfig(host, connectionConfig);
     }
 
-    class InternalPoolEntryCallback implements FutureCallback<CPoolEntry> {
-
-        private final BasicFuture<NHttpClientConnection> future;
-
-        public InternalPoolEntryCallback(
-                final BasicFuture<NHttpClientConnection> future) {
-            super();
-            this.future = future;
-        }
-
-        @Override
-        public void completed(final CPoolEntry entry) {
-            Asserts.check(entry.getConnection() != null, "Pool entry with no connection");
-            if (log.isDebugEnabled()) {
-                log.debug("Connection leased: " + format(entry) + formatStats(entry.getRoute()));
-            }
-            final NHttpClientConnection managedConn = CPoolProxy.newProxy(entry);
-            if (!this.future.completed(managedConn)) {
-                pool.release(entry, true);
-            }
-        }
-
-        @Override
-        public void failed(final Exception ex) {
-            if (log.isDebugEnabled()) {
-                log.debug("Connection request failed", ex);
-            }
-            this.future.failed(ex);
-        }
-
-        @Override
-        public void cancelled() {
-            log.debug("Connection request cancelled");
-            this.future.cancel(true);
-        }
-
-    }
-
     static class ConfigData {
 
         private final Map<HttpHost, ConnectionConfig> connectionConfigMap;
